/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __DOTNET_LAUNCHER_H__
#define __DOTNET_LAUNCHER_H__

#include "launcher.h"

extern "C"
{
	typedef int (*coreclr_initialize_ptr)(
			const char* exePath,
			const char* appDomainFriendlyName,
			int propertyCount,
			const char** propertyKeys,
			const char** propertyValues,
			void** hostHandle,
			unsigned int* domainId);

	typedef int (*coreclr_execute_assembly_ptr)(
			void* hostHandle,
			unsigned int domainId,
			int argc,
			const char** argv,
			const char* managedAssemblyPath,
			unsigned int* exitCode);

	typedef int (*coreclr_shutdown_ptr)(
			void* hostHandle,
			unsigned int domainId);

	typedef int (*coreclr_create_delegate_ptr)(
			void* hostHandle,
			unsigned int domainId,
			const char* entryPointAssemblyName,
			const char* entryPointTypeName,
			const char* entryPointMethodName,
			void** delegate);

	typedef void (*plugin_initialize_ptr)();

	typedef void (*plugin_preload_ptr)();

	typedef void (*plugin_set_app_info_ptr)(
			const char* appId,
			const char* hostHandle);

	typedef char* (*plugin_get_dll_path_ptr)();

	typedef void (*plugin_before_execute_ptr)();

	typedef void (*plugin_finalize_ptr)();
}

namespace tizen {
namespace runtime {
namespace dotnetcore {

typedef void (*PreparedFunctionPtr)();
typedef bool (*LaunchFunctionPtr)(const char* root, const char* path, int argc, char* argv[]);

class CoreRuntime : public tizen::runtime::LauncherInterface
{
	public:
		CoreRuntime();
		~CoreRuntime();
		int initialize(bool standalone) override;
		void dispose() override;
		int runManagedLauncher(const char* appId, const char* appBase, const char* tpaList) override;
		int launch(const char* appId, const char* root, const char* path, int argc, char* argv[]) override;

	private:
		bool initializeCoreClr(const char* appId, const char* assemblyProbePaths, const char* pinvokeProbePaths, const char* tpaList);
		coreclr_initialize_ptr initializeClr;
		coreclr_execute_assembly_ptr executeAssembly;
		coreclr_shutdown_ptr shutdown;
		coreclr_create_delegate_ptr createDelegate;
		std::string __deviceAPIDirectory;
		std::string __runtimeDirectory;
		std::string __launcherAssembly;
		std::string __nativeLibDirectory;
		std::string __refAPIDirectory;
		void* __coreclrLib;
		void* __hostHandle;
		unsigned int __domainId;
		PreparedFunctionPtr preparedFunction;
		LaunchFunctionPtr launchFunction;
		// plugin function pointer
		void* __pluginLib;
		plugin_initialize_ptr pluginInitialize;
		plugin_preload_ptr	pluginPreload;
		plugin_set_app_info_ptr	pluginSetAppInfo;
		plugin_get_dll_path_ptr	pluginGetDllPath;
		plugin_before_execute_ptr pluginBeforeExecute;
		plugin_finalize_ptr	pluginFinalize;
};

}  // dotnetcore
}  // namespace runtime
}  // namespace tizen

#endif /* __DOTNET_LAUNCHER_H__ */